from config import Config
from docker import DockerClient
from flask import Flask
from flask_apscheduler import APScheduler
from flask_assets import Environment
from flask_login import LoginManager
from flask_mail import Mail
from flask_migrate import Migrate
from flask_paranoid import Paranoid
from flask_socketio import SocketIO
from flask_sqlalchemy import SQLAlchemy
from flask_hashids import Hashids


assets = Environment()
db = SQLAlchemy()
docker_client = DockerClient()
hashids = Hashids()
login = LoginManager()
login.login_view = 'auth.login'
login.login_message = 'Please log in to access this page.'
mail = Mail()
migrate = Migrate()
paranoid = Paranoid()
paranoid.redirect_view = '/'
scheduler = APScheduler()
socketio = SocketIO()


def create_app(config: Config = Config) -> Flask:
    ''' Creates an initialized Flask (WSGI Application) object. '''
    app: Flask = Flask(__name__)
    app.config.from_object(config)
    config.init_app(app)
    docker_client.login(
        app.config['NOPAQUE_DOCKER_REGISTRY_USERNAME'],
        password=app.config['NOPAQUE_DOCKER_REGISTRY_PASSWORD'],
        registry=app.config['NOPAQUE_DOCKER_REGISTRY']
    )

    assets.init_app(app)
    db.init_app(app)
    hashids.init_app(app)
    login.init_app(app)
    mail.init_app(app)
    migrate.init_app(app, db)
    paranoid.init_app(app)
    scheduler.init_app(app)
    socketio.init_app(app, message_queue=app.config['NOPAQUE_SOCKETIO_MESSAGE_QUEUE_URI'])  # noqa

    from .admin import bp as admin_blueprint
    app.register_blueprint(admin_blueprint, url_prefix='/admin')

    from .auth import bp as auth_blueprint
    app.register_blueprint(auth_blueprint, url_prefix='/auth')

    from .contributions import bp as contributions_blueprint
    app.register_blueprint(contributions_blueprint, url_prefix='/contributions')

    from .corpora import bp as corpora_blueprint
    app.register_blueprint(corpora_blueprint, url_prefix='/corpora')

    from .errors import bp as errors_blueprint
    app.register_blueprint(errors_blueprint)

    from .jobs import bp as jobs_blueprint
    app.register_blueprint(jobs_blueprint, url_prefix='/jobs')

    from .main import bp as main_blueprint
    app.register_blueprint(main_blueprint, url_prefix='/')

    from .services import bp as services_blueprint
    app.register_blueprint(services_blueprint, url_prefix='/services')

    from .settings import bp as settings_blueprint
    app.register_blueprint(settings_blueprint, url_prefix='/settings')

    from .users import bp as users_blueprint
    app.register_blueprint(users_blueprint, url_prefix='/users')

    return app
